"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTokenModuleEntry = exports.createUserSubstoreArrayEntry = exports.createLegacyReserveAccount = exports.getLockedBalances = void 0;
const lisk_framework_1 = require("lisk-framework");
const constants_1 = require("../constants");
const AMOUNT_ZERO = BigInt('0');
const getLockedBalances = async (account) => {
    let amount = AMOUNT_ZERO;
    if (account) {
        for (const vote of account.dpos.sentVotes) {
            amount += BigInt(vote.amount);
        }
        for (const unlockingObj of account.dpos.unlocking) {
            amount += BigInt(unlockingObj.amount);
        }
    }
    if (amount > AMOUNT_ZERO) {
        return [{ module: constants_1.MODULE_NAME_POS, amount: String(amount) }];
    }
    return [];
};
exports.getLockedBalances = getLockedBalances;
const createLegacyReserveAccount = async (legacyReserveAccount, legacyReserveAmount, tokenID) => {
    const tokenIDBuffer = Buffer.from(tokenID, 'hex');
    const lockedBalances = legacyReserveAccount ? await (0, exports.getLockedBalances)(legacyReserveAccount) : [];
    lockedBalances.push({
        module: constants_1.MODULE_NAME_LEGACY,
        amount: String(legacyReserveAmount),
    });
    lockedBalances.sort((a, b) => a.module.localeCompare(b.module, 'en'));
    const legacyReserve = {
        address: constants_1.ADDRESS_LEGACY_RESERVE,
        tokenID: tokenIDBuffer,
        availableBalance: String(legacyReserveAccount ? legacyReserveAccount.token.balance : AMOUNT_ZERO),
        lockedBalances,
    };
    return legacyReserve;
};
exports.createLegacyReserveAccount = createLegacyReserveAccount;
const createUserSubstoreArrayEntry = async (account, tokenID) => {
    const tokenIDBuffer = Buffer.from(tokenID, 'hex');
    if (!constants_1.ADDRESS_LEGACY_RESERVE.equals(account.address)) {
        const lockedBalances = await (0, exports.getLockedBalances)(account);
        return {
            address: account.address,
            tokenID: tokenIDBuffer,
            availableBalance: String(account.token.balance),
            lockedBalances,
        };
    }
    return null;
};
exports.createUserSubstoreArrayEntry = createUserSubstoreArrayEntry;
const getTokenModuleEntry = async (userSubstore, supplySubstore, escrowSubstore, supportedTokensSubstore) => {
    const tokenObj = {
        userSubstore,
        supplySubstore,
        escrowSubstore,
        supportedTokensSubstore,
    };
    return {
        module: constants_1.MODULE_NAME_TOKEN,
        data: tokenObj,
        schema: lisk_framework_1.tokenGenesisStoreSchema,
    };
};
exports.getTokenModuleEntry = getTokenModuleEntry;
//# sourceMappingURL=token.js.map