"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.writeCommandsToExec = exports.getCommandsToExecPostMigration = void 0;
const path_1 = require("path");
const fs_1 = require("./fs");
const constants_1 = require("../constants");
const getCommandsToExecPostMigration = async (networkConstant, snapshotHeight, outputDir) => {
    var _a, _b, _c;
    const commandsToExecute = [];
    const forgingStatusJsonFilepath = (0, path_1.resolve)(outputDir, constants_1.FILE_NAME.FORGING_STATUS);
    if (await (0, fs_1.exists)(forgingStatusJsonFilepath)) {
        const forgingStatusString = (await (0, fs_1.read)(forgingStatusJsonFilepath));
        const forgingStatusJson = JSON.parse(forgingStatusString);
        const chainID = parseInt(networkConstant.tokenID.substring(0, 2), 16);
        for (const forgingStatus of forgingStatusJson) {
            commandsToExecute.push('\n', `## Generate/Register the BLS keys for validator ${forgingStatus.lskAddress} - Please modify the command if necessary`, '\n');
            const keysFilepath = (0, path_1.resolve)(outputDir, constants_1.FILE_NAME.KEYS);
            commandsToExecute.push(`lisk-core keys:create --chainid ${chainID} --output ${keysFilepath} --add-legacy`, `lisk-core keys:import --file-path ${keysFilepath}`, `lisk-core endpoint:invoke random_setHashOnion '{ "address":"${forgingStatus.lskAddress}"}'`, `lisk-core endpoint:invoke generator_setStatus '{ "address":"${forgingStatus.lskAddress}", "height": ${(_a = forgingStatus.height) !== null && _a !== void 0 ? _a : snapshotHeight}, "maxHeightGenerated":  ${(_b = forgingStatus.maxHeightPreviouslyForged) !== null && _b !== void 0 ? _b : snapshotHeight}, "maxHeightPrevoted":  ${(_c = forgingStatus.maxHeightPrevoted) !== null && _c !== void 0 ? _c : snapshotHeight} }' --pretty`, `lisk-core generator:enable ${forgingStatus.lskAddress} --use-status-value`, 'lisk-core transaction:create legacy registerKeys 400000 --key-derivation-path=legacy --send');
            commandsToExecute.push('\n', '-----------------------------------------------------', '\n');
        }
    }
    return commandsToExecute;
};
exports.getCommandsToExecPostMigration = getCommandsToExecPostMigration;
const writeCommandsToExec = async (_this, networkConstant, snapshotHeight, outputDir, preCompletionCommands) => {
    const commandsToExecPreCompletion = preCompletionCommands !== null && preCompletionCommands !== void 0 ? preCompletionCommands : [];
    const commandsToExecPostMigration = await (0, exports.getCommandsToExecPostMigration)(networkConstant, snapshotHeight, outputDir);
    const allCommandsToExec = [
        '## Please execute the following commands to finish the migration successfully:',
        '------------------------------------------------------------------------------',
        '\n',
        ...commandsToExecPreCompletion,
        ...commandsToExecPostMigration,
    ].join('\n');
    if (commandsToExecPostMigration.length || commandsToExecPreCompletion.length) {
        const commandsToExecuteFilepath = (0, path_1.resolve)(outputDir, constants_1.FILE_NAME.COMMANDS_TO_EXEC);
        _this.log(commandsToExecPreCompletion.length
            ? `Creating file with the list of commands to execute post migration: ${commandsToExecuteFilepath}`
            : `Creating file with the list of commands to execute: ${commandsToExecuteFilepath}`);
        await (0, fs_1.write)(commandsToExecuteFilepath, allCommandsToExec);
        _this.log(commandsToExecPreCompletion.length
            ? `Successfully created file with the list of commands to execute post migration: ${commandsToExecuteFilepath}`
            : `Successfully created file with the list of commands to execute: ${commandsToExecuteFilepath}`);
    }
};
exports.writeCommandsToExec = writeCommandsToExec;
//# sourceMappingURL=commands.js.map